@@ -31,38 +31,39 @@
  * \author Ian Romanick <idr@us.ibm.com>
  */
 
-#if defined(IN_MINI_GLX)
+#ifdef HAVE_DIX_CONFIG_H
+#include <dix-config.h>
+#endif
+
+#ifdef DRI_NEW_INTERFACE_ONLY
 # include <stdlib.h>
 # include <string.h>
 # include <GL/gl.h>
-# include "GL/internal/dri_interface.h"
+# include "dri_interface.h"
 # include "imports.h"
-# define _mesa_memset  memset
+# define __glXMemset  memset
 #else
-# if defined(HAVE_DIX_CONFIG_H)
-#  include <dix-config.h>
-# endif
 # include <X11/X.h>
 # include <GL/glx.h>
 # include "GL/glxint.h"
 
 # ifdef XFree86Server
-# include <os.h>
-# include <string.h>
-#  define _mesa_malloc(b) xalloc(b)
-#  define _mesa_free(m)   xfree(m)
-#  define _mesa_memset   memset
+#  include "GL/glx_ansic.h"
+extern void * __glXMalloc( size_t size );
+extern void __glXFree( void * ptr );
+#  define _mesa_malloc(b) __glXMalloc(b)
+#  define _mesa_free(m)   __glXFree(m)
 # else
 #  include <X11/Xlibint.h>
-#  define _mesa_memset memset
+#  define __glXMemset  memset
 #  define _mesa_malloc(b) Xmalloc(b)
 #  define _mesa_free(m) Xfree(m)
 # endif /* XFree86Server */
-#endif /* !defined(IN_MINI_GLX) */
+#endif  /* DRI_NEW_INTERFACE_ONLY */
 
 #include "glcontextmodes.h"
 
-#if !defined(IN_MINI_GLX)
+#ifndef DRI_NEW_INTERFACE_ONLY
 #define NUM_VISUAL_TYPES   6
 
 /**
@@ -104,7 +105,7 @@ _gl_convert_to_x_visual_type( int visualType )
 	GrayScale,   StaticGray
     };
 
-    return ( (unsigned) (visualType - GLX_TRUE_COLOR) < NUM_VISUAL_TYPES )
+    return ( (unsigned) (visualType - GLX_TRUE_COLOR) <= NUM_VISUAL_TYPES )
 	? x_visual_types[ visualType - GLX_TRUE_COLOR ] : -1;
 }
 
@@ -129,7 +130,7 @@ _gl_copy_visual_to_context_mode( __GLcontextModes * mode,
 {
     __GLcontextModes * const next = mode->next;
 
-    (void) _mesa_memset( mode, 0, sizeof( __GLcontextModes ) );
+    (void) __glXMemset( mode, 0, sizeof( __GLcontextModes ) );
     mode->next = next;
 
     mode->visualID = config->vid;
@@ -160,8 +161,8 @@ _gl_copy_visual_to_context_mode( __GLcontextModes * mode,
     mode->greenMask = config->greenMask;
     mode->blueMask = config->blueMask;
     mode->alphaMask = config->alphaMask;
-    mode->rgbBits = mode->rgbMode ? config->bufferSize : 0;
-    mode->indexBits = mode->colorIndexMode ? config->bufferSize : 0;
+    mode->rgbBits = config->bufferSize;
+    mode->indexBits = config->bufferSize;
 
     mode->accumRedBits = config->accumRedSize;
     mode->accumGreenBits = config->accumGreenSize;
@@ -182,15 +183,6 @@ _gl_copy_visual_to_context_mode( __GLcontextModes * mode,
     mode->transparentIndex = config->transparentIndex;
 
     mode->swapMethod = GLX_SWAP_UNDEFINED_OML;
-
-    mode->bindToTextureRgb = (mode->rgbMode) ? GL_TRUE : GL_FALSE;
-    mode->bindToTextureRgba = (mode->rgbMode && mode->alphaBits) ?
-	GL_TRUE : GL_FALSE;
-    mode->bindToMipmapTexture = mode->rgbMode ? GL_TRUE : GL_FALSE;
-    mode->bindToTextureTargets = mode->rgbMode ?
-	GLX_TEXTURE_1D_BIT_EXT | GLX_TEXTURE_2D_BIT_EXT |
-	GLX_TEXTURE_RECTANGLE_BIT_EXT : 0;
-    mode->yInverted = GL_FALSE;
 }
 
 
@@ -322,21 +314,6 @@ _gl_get_context_mode_data(const __GLcontextModes *mode, int attribute,
       case GLX_SAMPLES_SGIS:
 	*value_return = mode->samples;
 	return 0;
-      case GLX_BIND_TO_TEXTURE_RGB_EXT:
-	*value_return = mode->bindToTextureRgb;
-	return 0;
-      case GLX_BIND_TO_TEXTURE_RGBA_EXT:
-	*value_return = mode->bindToTextureRgba;
-	return 0;
-      case GLX_BIND_TO_MIPMAP_TEXTURE_EXT:
-	*value_return = mode->bindToMipmapTexture;
-	return 0;
-      case GLX_BIND_TO_TEXTURE_TARGETS_EXT:
-	*value_return = mode->bindToTextureTargets;
-	return 0;
-      case GLX_Y_INVERTED_EXT:
-	*value_return = mode->yInverted;
-	return 0;
 
       /* Applications are NOT allowed to query GLX_VISUAL_SELECT_GROUP_SGIX.
        * It is ONLY for communication between the GLX client and the GLX
@@ -347,7 +324,7 @@ _gl_get_context_mode_data(const __GLcontextModes *mode, int attribute,
 	return GLX_BAD_ATTRIBUTE;
     }
 }
-#endif /* !defined(IN_MINI_GLX) */
+#endif /* DRI_NEW_INTERFACE_ONLY */
 
 
 /**
@@ -394,7 +371,7 @@ _gl_context_modes_create( unsigned count, size_t minimum_size )
 	 break;
       }
       
-      (void) _mesa_memset( *next, 0, size );
+      (void) __glXMemset( *next, 0, size );
       (*next)->visualID = GLX_DONT_CARE;
       (*next)->visualType = GLX_DONT_CARE;
       (*next)->visualRating = GLX_NONE;
@@ -407,11 +384,6 @@ _gl_context_modes_create( unsigned count, size_t minimum_size )
       (*next)->xRenderable = GLX_DONT_CARE;
       (*next)->fbconfigID = GLX_DONT_CARE;
       (*next)->swapMethod = GLX_SWAP_UNDEFINED_OML;
-      (*next)->bindToTextureRgb = GLX_DONT_CARE;
-      (*next)->bindToTextureRgba = GLX_DONT_CARE;
-      (*next)->bindToMipmapTexture = GLX_DONT_CARE;
-      (*next)->bindToTextureTargets = 0;
-      (*next)->yInverted = GLX_DONT_CARE;
 
       next = & ((*next)->next);
    }
@@ -524,10 +496,5 @@ _gl_context_modes_are_same( const __GLcontextModes * a,
 	    (a->maxPbufferPixels == b->maxPbufferPixels) &&
 	    (a->optimalPbufferWidth == b->optimalPbufferWidth) &&
 	    (a->optimalPbufferHeight == b->optimalPbufferHeight) &&
-	    (a->swapMethod == b->swapMethod) &&
-	    (a->bindToTextureRgb == b->bindToTextureRgb) &&
-	    (a->bindToTextureRgba == b->bindToTextureRgba) &&
-	    (a->bindToMipmapTexture == b->bindToMipmapTexture) &&
-	    (a->bindToTextureTargets == b->bindToTextureTargets) &&
-	    (a->yInverted == b->yInverted) );
+	    (a->swapMethod == b->swapMethod) );
 }
