@@ -30,6 +30,7 @@ or other dealings in this Software without prior written authorization
 of the copyright holder.
 
 ************************************************************/
+/* $XdotOrg: xc/programs/Xserver/Xext/xevie.c,v 1.6 2005/05/22 01:12:49 alanc Exp $ */
 
 #define NEED_REPLIES
 #define NEED_EVENTS
@@ -52,19 +53,17 @@ of the copyright holder.
 #include "inputstr.h"
 #include "windowstr.h"
 #include "cursorstr.h"
-#include <xkbsrv.h>
+#include <X11/extensions/XKBsrv.h>
 
 #include "../os/osdep.h"
 
 #define NoSuchEvent 0x80000000
 
-#ifdef XKB
 extern Bool noXkbExtension;
-#endif
 extern int    xeviegrabState;
 
-static int		ProcDispatch (register ClientPtr client), SProcDispatch (register ClientPtr client);
-static void		ResetProc (ExtensionEntry *extEntry);
+static int		ProcDispatch (), SProcDispatch ();
+static void		ResetProc ();
 
 static unsigned char	ReqCode = 0;
 static int		ErrorBase;
@@ -76,11 +75,11 @@ DeviceIntPtr		xeviemouse = NULL;
 Mask			xevieMask = 0;
 int       		xevieEventSent = 0;
 int			xevieKBEventSent = 0;
-static DevPrivateKey    xevieDevicePrivateKey = &xevieDevicePrivateKey;
+static unsigned int             xevieServerGeneration;
+static int                      xevieDevicePrivateIndex;
 static Bool                     xevieModifiersOn = FALSE;
 
-#define XEVIEINFO(dev)  ((xevieDeviceInfoPtr) \
-    dixLookupPrivate(&(dev)->devPrivates, xevieDevicePrivateKey))
+#define XEVIEINFO(dev)  ((xevieDeviceInfoPtr)dev->devPrivates[xevieDevicePrivateIndex].ptr)
 
 Mask xevieFilters[128] = 
 {
@@ -105,7 +104,7 @@ typedef struct {
 } xevieKeycQueueRec, *xevieKeycQueuePtr;
 
 #define KEYC_QUEUE_SIZE	    100
-static xevieKeycQueueRec keycq[KEYC_QUEUE_SIZE] = {{0, NULL}};
+xevieKeycQueueRec keycq[KEYC_QUEUE_SIZE] = {0, NULL};
 static int keycqHead = 0, keycqTail = 0;
 
 static int              ProcDispatch (ClientPtr), SProcDispatch (ClientPtr);
@@ -130,10 +129,16 @@ static void XeviePointerProcessInputProc(xEvent *xE, DeviceIntPtr dev,
 static void XevieKbdProcessInputProc(xEvent *xE, DeviceIntPtr dev, int count);
 
 void
-XevieExtensionInit (void)
+XevieExtensionInit ()
 {
     ExtensionEntry* extEntry;
 
+    if (serverGeneration != xevieServerGeneration) {
+        if ((xevieDevicePrivateIndex = AllocateDevicePrivateIndex()) == -1)
+            return;
+        xevieServerGeneration = serverGeneration;
+    }
+
     if (!AddCallback(&ServerGrabCallback,XevieServerGrabStateCallback,NULL))
        return;
 
@@ -153,14 +158,18 @@ XevieExtensionInit (void)
 
 /*ARGSUSED*/
 static 
-void ResetProc (ExtensionEntry *extEntry)
+void ResetProc (extEntry)
+    ExtensionEntry* extEntry;
 {
 }
 
 static 
-int ProcQueryVersion (register ClientPtr client)
+int ProcQueryVersion (client)
+    register ClientPtr client;
 {
+    REQUEST (xXevieQueryVersionReq);
     xXevieQueryVersionReply rep;
+    register int n;
 
     REQUEST_SIZE_MATCH (xXevieQueryVersionReq);
     rep.type = X_Reply;
@@ -173,9 +182,12 @@ int ProcQueryVersion (register ClientPtr client)
 }
 
 static
-int ProcStart (register ClientPtr client)
+int ProcStart (client)
+    register ClientPtr client;
 {
+    REQUEST (xXevieStartReq);
     xXevieStartReply rep;
+    register int n;
 
     REQUEST_SIZE_MATCH (xXevieStartReq);
     rep.pad1 = 0;
@@ -196,14 +208,12 @@ int ProcStart (register ClientPtr client)
            return BadAlloc;
     } else
         return BadAccess;
-#ifdef XKB
     if (!noXkbExtension) {
 	if (!XevieStart()) {
             DeleteCallback(&ClientStateCallback,XevieClientStateCallback,NULL);
             return BadAlloc;
         }
     }
-#endif
     
     xevieModifiersOn = FALSE;
 
@@ -214,7 +224,8 @@ int ProcStart (register ClientPtr client)
 }
 
 static
-int ProcEnd (register ClientPtr client)
+int ProcEnd (client)
+    register ClientPtr client;
 {
     xXevieEndReply rep;
 
@@ -233,11 +244,13 @@ int ProcEnd (register ClientPtr client)
 }
 
 static
-int ProcSend (register ClientPtr client)
+int ProcSend (client)
+    register ClientPtr client;
 {
     REQUEST (xXevieSendReq);
     xXevieSendReply rep;
     xEvent *xE;
+    OsCommPtr oc;
     static unsigned char lastDetail = 0, lastType = 0;
 
     if (client->index != xevieClientIndex)
@@ -252,23 +265,19 @@ int ProcSend (register ClientPtr client)
 	case KeyPress:
         case KeyRelease:
 	  xevieKBEventSent = 1;
-#ifdef XKB
-          if(!noXkbExtension)
-	    doSendEvent(xE, inputInfo.keyboard);
-	  else 
-#endif
+          if(noXkbExtension)
             CoreProcessKeyboardEvent (xE, xeviekb, 1);
+	  else 
+	    doSendEvent(xE, inputInfo.keyboard);
 	  break;
 	case ButtonPress:
 	case ButtonRelease:
 	case MotionNotify:
 	  xevieEventSent = 1;
-#ifdef XKB
-	  if(!noXkbExtension)
-	    doSendEvent(xE, inputInfo.pointer);
-	  else
-#endif
+	  if(noXkbExtension)
 	    CoreProcessPointerEvent(xE, xeviemouse, 1); 
+	  else
+	    doSendEvent(xE, inputInfo.pointer);
 	  break; 
 	default:
 	  break;
@@ -279,7 +288,8 @@ int ProcSend (register ClientPtr client)
 }
 
 static
-int ProcSelectInput (register ClientPtr client)
+int ProcSelectInput (client)
+    register ClientPtr client;
 {
     REQUEST (xXevieSelectInputReq);
     xXevieSelectInputReply rep;
@@ -295,7 +305,8 @@ int ProcSelectInput (register ClientPtr client)
 }
 
 static 
-int ProcDispatch (register ClientPtr client)
+int ProcDispatch (client)
+    register ClientPtr	client;
 {
     REQUEST (xReq);
     switch (stuff->data)
@@ -316,7 +327,8 @@ int ProcDispatch (register ClientPtr client)
 }
 
 static 
-int SProcQueryVersion (register ClientPtr client)
+int SProcQueryVersion (client)
+    register ClientPtr	client;
 {
     register int n;
 
@@ -326,7 +338,8 @@ int SProcQueryVersion (register ClientPtr client)
 }
 
 static 
-int SProcStart (ClientPtr client)
+int SProcStart (client)
+    ClientPtr client;
 {
     register int n;
 
@@ -338,9 +351,12 @@ int SProcStart (ClientPtr client)
 }
 
 static 
-int SProcEnd (ClientPtr client)
+int SProcEnd (client)
+    ClientPtr client;
 {
     register int n;
+    int count;
+    xColorItem* pItem;
 
     REQUEST (xXevieEndReq);
     swaps (&stuff->length, n);
@@ -350,9 +366,11 @@ int SProcEnd (ClientPtr client)
 }
 
 static
-int SProcSend (ClientPtr client)
+int SProcSend (client)
+    ClientPtr client;
 {
     register int n;
+    int count;
 
     REQUEST (xXevieSendReq);
     swaps (&stuff->length, n);
@@ -362,20 +380,23 @@ int SProcSend (ClientPtr client)
 }
 
 static
-int SProcSelectInput (ClientPtr client)
+int SProcSelectInput (client)
+    ClientPtr client;
 {
     register int n;
+    int count;
 
     REQUEST (xXevieSelectInputReq);
     swaps (&stuff->length, n);
-    REQUEST_AT_LEAST_SIZE (xXevieSelectInputReq);
+    REQUEST_AT_LEAST_SIZE (xXevieSendReq);
     swapl(&stuff->event_mask, n);
     return ProcSelectInput (client);
 }
 
 
 static 
-int SProcDispatch (register ClientPtr client)
+int SProcDispatch (client)
+    register ClientPtr	client;
 {
     REQUEST(xReq);
     switch (stuff->data)
@@ -465,7 +486,6 @@ XevieKbdProcessInputProc(xEvent *xE, DeviceIntPtr dev, int count)
         xE->u.keyButtonPointer.state = keyc->state | inputInfo.pointer->button->state;
         /* fix bug: sequence lost in Xlib */
         xE->u.u.sequenceNumber = clients[xevieClientIndex]->sequence;
-#ifdef XKB
 	/* fix for bug5092586 */
 	if(!noXkbExtension) {
           switch(xE->u.u.type) {
@@ -473,7 +493,6 @@ XevieKbdProcessInputProc(xEvent *xE, DeviceIntPtr dev, int count)
 	    case KeyRelease: *kptr &= ~bit; break;
 	  }
 	}
-#endif
 	keycq[keycqHead].time = xE->u.keyButtonPointer.time;
 	memcpy(keycq[keycqHead].keyc, keyc, sizeof(KeyClassRec) - sizeof(KeyClassPtr));
 	memcpy(keycq[keycqHead].keyc->xkbInfo, keyc->xkbInfo, sizeof(XkbSrvInfoRec));
@@ -528,7 +547,6 @@ XevieEnd(int clientIndex)
 {
     if (!clientIndex || clientIndex == xevieClientIndex) {
 
-#ifdef XKB
        if(!noXkbExtension) {
 
 	   XevieRemove(inputInfo.keyboard,NULL);
@@ -544,7 +562,6 @@ XevieEnd(int clientIndex)
            inputInfo.pointer->public.realInputProc = CoreProcessPointerEvent;
            XkbSetExtension(inputInfo.pointer,ProcessPointerEvent);
        }
-#endif
 
        xevieFlag = 0;
        xevieClientIndex = 0;
@@ -612,11 +629,14 @@ XevieAdd(DeviceIntPtr device, void* data)
 {
     xevieDeviceInfoPtr xeviep;
 
+    if (!AllocateDevicePrivate(device, xevieDevicePrivateIndex))
+        return FALSE;
+
     xeviep = xalloc (sizeof (xevieDeviceInfoRec));
     if (!xeviep)
             return FALSE;
 
-    dixSetPrivate(&device->devPrivates, xevieDevicePrivateKey, xeviep);
+    device->devPrivates[xevieDevicePrivateIndex].ptr = xeviep;
     XevieUnwrapAdd(device, data);
 
     return TRUE;
@@ -633,7 +653,7 @@ XevieRemove(DeviceIntPtr device,pointer data)
     UNWRAP_UNWRAPPROC(device,xeviep->unwrapProc);
 
     xfree(xeviep);
-    dixSetPrivate(&device->devPrivates, xevieDevicePrivateKey, NULL);
+    device->devPrivates[xevieDevicePrivateIndex].ptr = NULL;
     return TRUE;
 }
 
@@ -676,13 +696,9 @@ doSendEvent(xEvent *xE, DeviceIntPtr dev)
 	}
         dev->key->modifierMap[xE->u.u.detail] = 0;  
 
-	if(dev->key->xkbInfo->repeatKey != 0 && xE->u.u.type != KeyPress)
-            XkbLastRepeatEvent=     (pointer)xE;
         UNWRAP_INPUTPROC(dev,xeviep);
         dev->public.processInputProc(xE,dev,1);
         COND_WRAP_INPUTPROC(dev,xeviep,tmp);
-        XkbLastRepeatEvent= NULL;
-
         dev->key->modifierMap[xE->u.u.detail] = realModes;
 	dev->key = keyc;
 	if(notFound) {
