@@ -1,3 +1,5 @@
+/* $Xorg: chgprop.c,v 1.4 2001/02/09 02:04:33 xorgcvs Exp $ */
+
 /************************************************************
 
 Copyright 1989, 1998  The Open Group
@@ -43,6 +45,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ********************************************************/
+/* $XFree86: xc/programs/Xserver/Xi/chgprop.c,v 3.2 2001/01/17 22:13:24 dawes Exp $ */
 
 /***********************************************************************
  *
@@ -50,14 +53,20 @@ SOFTWARE.
  *
  */
 
+#define	 NEED_EVENTS
+#define	 NEED_REPLIES
 #ifdef HAVE_DIX_CONFIG_H
 #include <dix-config.h>
 #endif
 
-#include "inputstr.h"	/* DeviceIntPtr      */
+#include <X11/X.h>				/* for inputstr.h    */
+#include <X11/Xproto.h>			/* Request macro     */
+#include "inputstr.h"			/* DeviceIntPtr	     */
 #include "windowstr.h"
 #include <X11/extensions/XI.h>
 #include <X11/extensions/XIproto.h>
+#include "extnsionst.h"
+#include "extinit.h"			/* LookupDeviceIntRec */
 
 #include "exevents.h"
 #include "exglobals.h"
@@ -72,20 +81,26 @@ SOFTWARE.
  */
 
 int
-SProcXChangeDeviceDontPropagateList(ClientPtr client)
-{
-    char n;
+SProcXChangeDeviceDontPropagateList(client)
+    register ClientPtr client;
+    {
+    register char n;
+    register long *p;
+    register int i;
 
     REQUEST(xChangeDeviceDontPropagateListReq);
     swaps(&stuff->length, n);
     REQUEST_AT_LEAST_SIZE(xChangeDeviceDontPropagateListReq);
     swapl(&stuff->window, n);
     swaps(&stuff->count, n);
-    REQUEST_FIXED_SIZE(xChangeDeviceDontPropagateListReq,
-                      stuff->count * sizeof(CARD32));
-    SwapLongs((CARD32 *) (&stuff[1]), stuff->count);
-    return (ProcXChangeDeviceDontPropagateList(client));
-}
+    p = (long *) &stuff[1];
+    for (i=0; i<stuff->count; i++)
+        {
+        swapl(p, n);
+	p++;
+        }
+    return(ProcXChangeDeviceDontPropagateList(client));
+    }
 
 /***********************************************************************
  *
@@ -94,38 +109,51 @@ SProcXChangeDeviceDontPropagateList(ClientPtr client)
  */
 
 int
-ProcXChangeDeviceDontPropagateList(ClientPtr client)
-{
-    int i, rc;
-    WindowPtr pWin;
-    struct tmask tmp[EMASKSIZE];
-    OtherInputMasks *others;
+ProcXChangeDeviceDontPropagateList (client)
+    register ClientPtr client;
+    {
+    int			i;
+    WindowPtr		pWin;
+    struct 		tmask tmp[EMASKSIZE];
+    OtherInputMasks	*others;
 
     REQUEST(xChangeDeviceDontPropagateListReq);
     REQUEST_AT_LEAST_SIZE(xChangeDeviceDontPropagateListReq);
 
-    if (stuff->length != bytes_to_int32(sizeof(xChangeDeviceDontPropagateListReq)) +
+    if (stuff->length !=(sizeof(xChangeDeviceDontPropagateListReq)>>2) + 
 	stuff->count)
-	return BadLength;
+	{
+	SendErrorToClient (client, IReqCode, X_ChangeDeviceDontPropagateList, 0,
+	    BadLength);
+	return Success;
+	}
 
-    rc = dixLookupWindow(&pWin, stuff->window, client, DixSetAttrAccess);
-    if (rc != Success)
-	return rc;
-
-    if (stuff->mode != AddToList && stuff->mode != DeleteFromList) {
+    pWin = (WindowPtr) LookupWindow (stuff->window, client);
+    if (!pWin)
+        {
 	client->errorValue = stuff->window;
-	return BadMode;
-    }
+	SendErrorToClient(client, IReqCode, X_ChangeDeviceDontPropagateList, 0, 
+		BadWindow);
+	return Success;
+        }
 
-    if ((rc = CreateMaskFromList(client, (XEventClass *) & stuff[1],
-				 stuff->count, tmp, NULL,
-				 X_ChangeDeviceDontPropagateList)) != Success)
-	return rc;
+    if (stuff->mode != AddToList && stuff->mode != DeleteFromList)
+        {
+	client->errorValue = stuff->window;
+	SendErrorToClient(client, IReqCode, X_ChangeDeviceDontPropagateList, 0, 
+		BadMode);
+	return Success;
+        }
+
+    if (CreateMaskFromList (client, (XEventClass *)&stuff[1], 
+	stuff->count, tmp, NULL, X_ChangeDeviceDontPropagateList) != Success)
+	    return Success;
 
     others = wOtherInputMasks(pWin);
     if (!others && stuff->mode == DeleteFromList)
 	return Success;
-    for (i = 0; i < EMASKSIZE; i++) {
+    for (i=0; i<EMASKSIZE; i++)
+	{
 	if (tmp[i].mask == 0)
 	    continue;
 
@@ -134,10 +162,13 @@ ProcXChangeDeviceDontPropagateList(ClientPtr client)
 	else if (others)
 	    tmp[i].mask |= others->dontPropagateMask[i];
 
-	if (DeviceEventSuppressForWindow(pWin, client, tmp[i].mask, i) !=
-	    Success)
-	    return BadClass;
-    }
+	if (DeviceEventSuppressForWindow (pWin,client,tmp[i].mask,i) != Success)
+	    {
+	    SendErrorToClient ( client, IReqCode, X_ChangeDeviceDontPropagateList, 0, 
+		BadClass);
+	    return Success;
+	    }
+	}
 
     return Success;
-}
+    }
