@@ -1,4 +1,6 @@
 /*
+ * Id: fbfill.c,v 1.1 1999/11/02 03:54:45 keithp Exp $
+ *
  * Copyright © 1998 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -19,12 +21,14 @@
  * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  */
+/* $XFree86: xc/programs/Xserver/fb/fbfill.c,v 1.5 2003/01/29 00:43:33 torrey Exp $ */
 
 #ifdef HAVE_DIX_CONFIG_H
 #include <dix-config.h>
 #endif
 
 #include "fb.h"
+#include "fbmmx.h"
 
 void
 fbFill (DrawablePtr pDrawable,
@@ -44,18 +48,17 @@ fbFill (DrawablePtr pDrawable,
 
     switch (pGC->fillStyle) {
     case FillSolid:
-#ifndef FB_ACCESS_WRAPPER
-	if (pPriv->and || !pixman_fill ((uint32_t *)dst, dstStride, dstBpp,
-					x + dstXoff, y + dstYoff,
-					width, height,
-					pPriv->xor))
+#ifdef USE_MMX
+	if (!pPriv->and && fbHaveMMX())
+	    if (fbSolidFillmmx (pDrawable, x, y, width, height, pPriv->xor))
+		return;
 #endif	    
-	    fbSolid (dst + (y + dstYoff) * dstStride, 
-		     dstStride, 
-		     (x + dstXoff) * dstBpp,
-		     dstBpp,
-		     width * dstBpp, height,
-		     pPriv->and, pPriv->xor);
+	fbSolid (dst + (y + dstYoff) * dstStride, 
+		 dstStride, 
+		 (x + dstXoff) * dstBpp,
+		 dstBpp,
+		 width * dstBpp, height,
+		 pPriv->and, pPriv->xor);
 	break;
     case FillStippled:
     case FillOpaqueStippled: {
@@ -69,7 +72,7 @@ fbFill (DrawablePtr pDrawable,
 	    FbBits	*stip;
 	    FbStride    stipStride;
 	    int		stipBpp;
-	    _X_UNUSED int	stipXoff, stipYoff;
+	    int		stipXoff, stipYoff; /* XXX assumed to be zero */
 
 	    if (pGC->fillStyle == FillStippled)
 		alu = FbStipple1Rop(pGC->alu,pGC->fgPixel);
@@ -89,15 +92,14 @@ fbFill (DrawablePtr pDrawable,
 		    dstBpp,
 		    
 		    (pGC->patOrg.x + pDrawable->x + dstXoff),
-		    pGC->patOrg.y + pDrawable->y - y);
-	    fbFinishAccess (&pStip->drawable);
+		    pGC->patOrg.y + pDrawable->y + dstYoff - y);
 	}
 	else
 	{
 	    FbStip	*stip;
 	    FbStride    stipStride;
 	    int		stipBpp;
-	    _X_UNUSED int	stipXoff, stipYoff;
+	    int		stipXoff, stipYoff; /* XXX assumed to be zero */
 	    FbBits	fgand, fgxor, bgand, bgxor;
 
 	    fgand = pPriv->and;
@@ -127,8 +129,7 @@ fbFill (DrawablePtr pDrawable,
 		       fgand, fgxor,
 		       bgand, bgxor,
 		       pGC->patOrg.x + pDrawable->x + dstXoff,
-		       pGC->patOrg.y + pDrawable->y - y);
-	    fbFinishAccess (&pStip->drawable);
+		       pGC->patOrg.y + pDrawable->y + dstYoff - y);
 	}
 	break;
     }
@@ -139,7 +140,7 @@ fbFill (DrawablePtr pDrawable,
 	int	    tileBpp;
 	int	    tileWidth;
 	int	    tileHeight;
-	_X_UNUSED int	tileXoff, tileYoff;
+	int	    tileXoff, tileYoff; /* XXX assumed to be zero */
 	
 	fbGetDrawable (&pTile->drawable, tile, tileStride, tileBpp, tileXoff, tileYoff);
 	tileWidth = pTile->drawable.width;
@@ -156,13 +157,11 @@ fbFill (DrawablePtr pDrawable,
 		pPriv->pm,
 		dstBpp,
 		(pGC->patOrg.x + pDrawable->x + dstXoff) * dstBpp,
-		pGC->patOrg.y + pDrawable->y - y);
-	fbFinishAccess (&pTile->drawable);
+		pGC->patOrg.y + pDrawable->y + dstYoff - y);
 	break;
     }
     }
     fbValidateDrawable (pDrawable);
-    fbFinishAccess (pDrawable);
 }
 
 void
@@ -185,7 +184,7 @@ fbSolidBoxClipped (DrawablePtr	pDrawable,
 
     fbGetDrawable (pDrawable, dst, dstStride, dstBpp, dstXoff, dstYoff);
     
-    for (nbox = RegionNumRects(pClip), pbox = RegionRects(pClip);
+    for (nbox = REGION_NUM_RECTS(pClip), pbox = REGION_RECTS(pClip); 
 	 nbox--; 
 	 pbox++)
     {
@@ -210,21 +209,14 @@ fbSolidBoxClipped (DrawablePtr	pDrawable,
 	
 	if (partY2 <= partY1)
 	    continue;
+	
+	fbSolid (dst + (partY1 + dstYoff) * dstStride,
+		 dstStride,
+		 (partX1 + dstXoff) * dstBpp,
+		 dstBpp,
 
-#ifndef FB_ACCESS_WRAPPER
-	if (and || !pixman_fill ((uint32_t *)dst, dstStride, dstBpp,
-				 partX1 + dstXoff, partY1 + dstYoff,
-				 (partX2 - partX1), (partY2 - partY1),
-				 xor))
-#endif
-	    fbSolid (dst + (partY1 + dstYoff) * dstStride,
-		     dstStride,
-		     (partX1 + dstXoff) * dstBpp,
-		     dstBpp,
-		     
-		     (partX2 - partX1) * dstBpp,
-		     (partY2 - partY1),
-		     and, xor);
+		 (partX2 - partX1) * dstBpp,
+		 (partY2 - partY1),
+		 and, xor);
     }
-    fbFinishAccess (pDrawable);
 }
