@@ -1,3 +1,5 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Module.h,v 1.37 2003/08/24 17:36:54 dawes Exp $ */
+
 /*
  * Copyright (c) 1997-2003 by The XFree86 Project, Inc.
  *
@@ -38,24 +40,31 @@
  * XXX This file arguably belongs in xfree86/loader/.
  */
 
+#ifdef HAVE_XORG_CONFIG_H
+#include <xorg-config.h>
+#endif
+
 #ifndef _XF86MODULE_H
 #define _XF86MODULE_H
 
 #include "misc.h"
-#include "extension.h"
+#include "xf86Version.h"
 #ifndef NULL
 #define NULL ((void *)0)
 #endif
 
 typedef enum {
-    LD_RESOLV_IFDONE = 0,       /* only check if no more
-                                   delays pending */
-    LD_RESOLV_NOW = 1,          /* finish one delay step */
-    LD_RESOLV_FORCE = 2         /* force checking... */
+    LD_RESOLV_IFDONE		= 0,	/* only check if no more
+					   delays pending */
+    LD_RESOLV_NOW		= 1,	/* finish one delay step */
+    LD_RESOLV_FORCE		= 2	/* force checking... */
 } LoaderResolveOptions;
 
 #define DEFAULT_LIST ((char *)-1)
 
+/* This indicates a special module that doesn't have the usual entry point */
+#define EXTERN_MODULE ((pointer)-1)
+
 /* Built-in ABI classes.  These definitions must not be changed. */
 #define ABI_CLASS_NONE		NULL
 #define ABI_CLASS_ANSIC		"X.Org ANSI C Emulation"
@@ -79,37 +88,41 @@ typedef enum {
  * changed.  The minor revision mask is 0x0000FFFF and the major revision
  * mask is 0xFFFF0000.
  */
-#define ABI_ANSIC_VERSION	SET_ABI_VERSION(0, 4)
-#define ABI_VIDEODRV_VERSION	SET_ABI_VERSION(17, 0)
-#define ABI_XINPUT_VERSION	SET_ABI_VERSION(21, 0)
-#define ABI_EXTENSION_VERSION	SET_ABI_VERSION(8, 0)
-#define ABI_FONT_VERSION	SET_ABI_VERSION(0, 6)
+#define ABI_ANSIC_VERSION	SET_ABI_VERSION(0, 2)
+#define ABI_VIDEODRV_VERSION	SET_ABI_VERSION(0, 7)
+#define ABI_XINPUT_VERSION	SET_ABI_VERSION(0, 4)
+#define ABI_EXTENSION_VERSION	SET_ABI_VERSION(0, 2)
+#define ABI_FONT_VERSION	SET_ABI_VERSION(0, 4)
 
 #define MODINFOSTRING1	0xef23fdc5
 #define MODINFOSTRING2	0x10dc023a
 
 #ifndef MODULEVENDORSTRING
+#ifndef __OS2ELF__
 #define MODULEVENDORSTRING	"X.Org Foundation"
+#else
+#define MODULEVENDORSTRING	"X.Org Foundation - OS2"
+#endif
 #endif
 
 /* Error return codes for errmaj.  New codes must only be added at the end. */
 typedef enum {
     LDR_NOERROR = 0,
-    LDR_NOMEM,                  /* memory allocation failed */
-    LDR_NOENT,                  /* Module file does not exist */
-    LDR_NOSUBENT,               /* pre-requsite file to be sub-loaded does not exist */
-    LDR_NOSPACE,                /* internal module array full */
-    LDR_NOMODOPEN,              /* module file could not be opened (check errmin) */
-    LDR_UNKTYPE,                /* file is not a recognized module type */
-    LDR_NOLOAD,                 /* type specific loader failed */
-    LDR_ONCEONLY,               /* Module should only be loaded once (not an error) */
-    LDR_NOPORTOPEN,             /* could not open port (check errmin) */
-    LDR_NOHARDWARE,             /* could not query/initialize the hardware device */
-    LDR_MISMATCH,               /* the module didn't match the spec'd requirments */
-    LDR_BADUSAGE,               /* LoadModule is called with bad arguments */
-    LDR_INVALID,                /* The module doesn't have a valid ModuleData object */
-    LDR_BADOS,                  /* The module doesn't support the OS */
-    LDR_MODSPECIFIC             /* A module-specific error in the SetupProc */
+    LDR_NOMEM,		/* memory allocation failed */
+    LDR_NOENT,		/* Module file does not exist */
+    LDR_NOSUBENT,	/* pre-requsite file to be sub-loaded does not exist */
+    LDR_NOSPACE,	/* internal module array full */
+    LDR_NOMODOPEN,	/* module file could not be opened (check errmin) */
+    LDR_UNKTYPE,	/* file is not a recognized module type */
+    LDR_NOLOAD,		/* type specific loader failed */
+    LDR_ONCEONLY,	/* Module should only be loaded once (not an error) */
+    LDR_NOPORTOPEN,	/* could not open port (check errmin) */
+    LDR_NOHARDWARE,	/* could not query/initialize the hardware device */
+    LDR_MISMATCH,	/* the module didn't match the spec'd requirments */
+    LDR_BADUSAGE,	/* LoadModule is called with bad arguments */
+    LDR_INVALID,	/* The module doesn't have a valid ModuleData object */
+    LDR_BADOS,		/* The module doesn't support the OS */
+    LDR_MODSPECIFIC	/* A module-specific error in the SetupProc */
 } LoaderErrorCode;
 
 /*
@@ -127,19 +140,19 @@ typedef enum {
 
 /* This structure is expected to be returned by the initfunc */
 typedef struct {
-    const char *modname;        /* name of module, e.g. "foo" */
-    const char *vendor;         /* vendor specific string */
-    CARD32 _modinfo1_;          /* constant MODINFOSTRING1/2 to find */
-    CARD32 _modinfo2_;          /* infoarea with a binary editor or sign tool */
-    CARD32 xf86version;         /* contains XF86_VERSION_CURRENT */
-    CARD8 majorversion;         /* module-specific major version */
-    CARD8 minorversion;         /* module-specific minor version */
-    CARD16 patchlevel;          /* module-specific patch level */
-    const char *abiclass;       /* ABI class that the module uses */
-    CARD32 abiversion;          /* ABI version */
-    const char *moduleclass;    /* module class description */
-    CARD32 checksum[4];         /* contains a digital signature of the */
-    /* version info structure */
+    const char * modname;	/* name of module, e.g. "foo" */
+    const char * vendor;	/* vendor specific string */
+    CARD32	 _modinfo1_;	/* constant MODINFOSTRING1/2 to find */
+    CARD32	 _modinfo2_;	/* infoarea with a binary editor or sign tool */
+    CARD32	 xf86version;	/* contains XF86_VERSION_CURRENT */
+    CARD8	 majorversion;	/* module-specific major version */
+    CARD8	 minorversion;	/* module-specific minor version */
+    CARD16	 patchlevel;	/* module-specific patch level */
+    const char * abiclass;	/* ABI class that the module uses */
+    CARD32	 abiversion;	/* ABI version */
+    const char * moduleclass;	/* module class description */
+    CARD32	 checksum[4];	/* contains a digital signature of the */
+				/* version info structure */
 } XF86ModuleVersionInfo;
 
 /*
@@ -147,12 +160,12 @@ typedef struct {
  * specify version and/or ABI requirements.
  */
 typedef struct {
-    CARD8 majorversion;         /* module-specific major version */
-    CARD8 minorversion;         /* moudle-specific minor version */
-    CARD16 patchlevel;          /* module-specific patch level */
-    const char *abiclass;       /* ABI class that the module uses */
-    CARD32 abiversion;          /* ABI version */
-    const char *moduleclass;    /* module class */
+    CARD8	 majorversion;	/* module-specific major version */
+    CARD8	 minorversion;	/* moudle-specific minor version */
+    CARD16	 patchlevel;	/* module-specific patch level */
+    const char * abiclass;	/* ABI class that the module uses */
+    CARD32	 abiversion;	/* ABI version */
+    const char * moduleclass;	/* module class */
 } XF86ModReqInfo;
 
 /* values to indicate unspecified fields in XF86ModReqInfo. */
@@ -169,31 +182,49 @@ typedef struct {
 
 #define INITARGS void
 
-/* Prototypes for Loader functions that are exported to modules */
-extern _X_EXPORT void *LoadSubModule(void *, const char *, const char **,
-                                       const char **, void *,
-                                       const XF86ModReqInfo *, int *, int *);
-extern _X_EXPORT void UnloadSubModule(void *);
-extern _X_EXPORT void UnloadModule(void *);
-extern _X_EXPORT void *LoaderSymbol(const char *);
-extern _X_EXPORT const char **LoaderListDirs(const char **, const char **);
-extern _X_EXPORT void LoaderFreeDirList(char **);
-extern _X_EXPORT void LoaderErrorMsg(const char *, const char *, int, int);
-extern _X_EXPORT void LoaderGetOS(const char **name, int *major, int *minor,
-                                  int *teeny);
-extern _X_EXPORT Bool LoaderShouldIgnoreABI(void);
-extern _X_EXPORT int LoaderGetABIVersion(const char *abiclass);
-
-typedef void *(*ModuleSetupProc) (void *, void *, int *, int *);
-typedef void (*ModuleTearDownProc) (void *);
-
-#define MODULESETUPPROTO(func) void *func(void *, void *, int*, int*)
-#define MODULETEARDOWNPROTO(func) void func(void *)
+typedef void (*InitExtension)(INITARGS);
 
 typedef struct {
-    XF86ModuleVersionInfo *vers;
-    ModuleSetupProc setup;
-    ModuleTearDownProc teardown;
+    InitExtension	initFunc;
+    const char *	name;
+    Bool		*disablePtr;
+    InitExtension	setupFunc;	
+    const char **	initDependencies;
+} ExtensionModule;
+
+extern ExtensionModule *ExtensionModuleList;
+
+/* Prototypes for Loader functions that are exported to modules */
+#ifndef IN_LOADER
+/* Prototypes with opaque pointers for use by modules */
+pointer LoadSubModule(pointer, const char *, const char **,
+		      const char **, pointer, const XF86ModReqInfo *,
+		      int *, int *);
+void UnloadSubModule(pointer);
+void LoadFont(pointer);
+void UnloadModule (pointer);
+#endif
+pointer LoaderSymbol(const char *);
+char **LoaderListDirs(const char **, const char **);
+void LoaderFreeDirList(char **);
+void LoaderErrorMsg(const char *, const char *, int, int);
+void LoadExtension(ExtensionModule *, Bool);
+void LoaderRefSymLists(const char **, ...);
+void LoaderRefSymbols(const char *, ...);
+void LoaderReqSymLists(const char **, ...);
+void LoaderReqSymbols(const char *, ...);
+int LoaderCheckUnresolved(int);
+void LoaderGetOS(const char **name, int *major, int *minor, int *teeny);
+
+typedef pointer (*ModuleSetupProc)(pointer, pointer, int *, int *);
+typedef void (*ModuleTearDownProc)(pointer);
+#define MODULESETUPPROTO(func) pointer func(pointer, pointer, int*, int*)
+#define MODULETEARDOWNPROTO(func) void func(pointer)
+
+typedef struct {
+    XF86ModuleVersionInfo *	vers;
+    ModuleSetupProc		setup;
+    ModuleTearDownProc		teardown;
 } XF86ModuleData;
 
-#endif                          /* _XF86STR_H */
+#endif /* _XF86STR_H */
