@@ -2,9 +2,6 @@
 #include <xorg-config.h>
 #endif
 
-#include <string.h>
-#include <unistd.h>
-
 #include "xf86.h"
 #include "xf86i2c.h"
 #include "msp3430.h"
@@ -20,7 +17,7 @@
 void InitMSP34xxG(MSP3430Ptr m);
 void InitMSP34x5D(MSP3430Ptr m);
 void CheckModeMSP34x5D(MSP3430Ptr m);
-static const char *MSP_getProductName (CARD16 product_id);
+char *MSP_getProductName (CARD16 product_id);
 void mpause(int milliseconds);
 
 #define __MSPDEBUG__	0
@@ -82,7 +79,6 @@ static void GetMSP3430Data(MSP3430Ptr m, CARD8 RegAddress, CARD8 RegSubAddressHi
    *RegValueLow = receive[1];
 }
 
-#if __MSPDEBUG__ > 2
 static void MSP3430DumpStatus(MSP3430Ptr m)
 {
 CARD8 status_hi, status_lo;
@@ -103,7 +99,6 @@ I2C_WriteRead(&(m->d), &subaddr, 1, data, 2);
 xf86DrvMsg(m->d.pI2CBus->scrnIndex, X_INFO, "MSP34xx: control=0x%02x%02x\n",
 		data[1], data[0]);
 }
-#endif
 
 /* wrapper */
 void InitMSP3430(MSP3430Ptr m)
@@ -136,7 +131,7 @@ MSP3430Ptr DetectMSP3430(I2CBusPtr b, I2CSlaveAddr addr)
    CARD8 hardware_version, major_revision, product_code, rom_version;
    Bool supported;
    
-   m = calloc(1,sizeof(MSP3430Rec));
+   m = xcalloc(1,sizeof(MSP3430Rec));
    if(m == NULL)return NULL;
    m->d.DevName = strdup("MSP34xx");
    m->d.SlaveAddr = addr;
@@ -149,8 +144,8 @@ MSP3430Ptr DetectMSP3430(I2CBusPtr b, I2CSlaveAddr addr)
    
    if(!I2C_WriteRead(&(m->d), NULL, 0, &a, 1))
    {
-       free(m->d.DevName);
-       free(m);
+   	xfree(m->d.DevName);
+   	xfree(m);
 	return NULL;
     }
 
@@ -235,14 +230,14 @@ MSP3430Ptr DetectMSP3430(I2CBusPtr b, I2CSlaveAddr addr)
 		MSP_getProductName(m->chip_id), supported?"":" (unsupported)", rom_version, m->chip_id);
 
 	if (!supported) {
-            free(m->d.DevName);
-            free(m);
-            return NULL;
-	}
+       		xfree(m->d.DevName);
+	   	xfree(m);
+		return NULL;
+	}	
    if(!I2CDevInit(&(m->d)))
    {
-       free(m->d.DevName);
-       free(m);
+       xfree(m->d.DevName);
+       xfree(m);
        return NULL;
    }
 
@@ -265,8 +260,9 @@ void ResetMSP3430(MSP3430Ptr m)
 void MSP3430SetVolume (MSP3430Ptr m, CARD8 value)
 {
     CARD8 result;
-#if 0
     CARD8 old_volume;
+
+#if 0
     GetMSP3430Data(m, RD_DSP, 0x00, 0x00, &old_volume, &result);   
     xf86DrvMsg(m->d.pI2CBus->scrnIndex, X_INFO, "MSP3430 result 0x%02x\n", result);
 #endif
@@ -305,8 +301,7 @@ void MSP3430SetSource(MSP3430Ptr m, CARD8 value)
 #endif
 
 
-static const char *
-MSP_getProductName (CARD16 product_id)
+char *MSP_getProductName (CARD16 product_id)
 {
 	switch (product_id) {
 		case 0x0400: return "MSP3400D";
@@ -610,6 +605,7 @@ void CheckModeMSP34x5D(MSP3430Ptr m) {
     const char dual_off=-stereo_off;
     char detect;
     CARD8 matrix, fmmatrix, source, high, low;
+    char *msg;
 
     fmmatrix=0;		/*no matrix*/
     source=0;		/*FM*/
@@ -703,7 +699,6 @@ void CheckModeMSP34x5D(MSP3430Ptr m) {
 			SetMSP3430Data (m, WR_DEM, 0x00, 0x21, 0, 1);
 
 #if __MSPDEBUG__ > 0
-		    char *msg;
 		    switch (matrix) {
 			case 0x30: /*MONO*/
 			    msg="MONO";
