@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/int10/pci.c,v 1.11 2001/10/01 13:44:13 eich Exp $ */
 
 /*
  *                   XFree86 int10 module
@@ -8,10 +9,9 @@
 #include <xorg-config.h>
 #endif
 
-#include <string.h>
-
 #include "xf86Pci.h"
 #include "xf86.h"
+#include "xf86_ansic.h"
 #define _INT10_PRIVATE
 #include "xf86int10.h"
 
