@@ -31,32 +31,50 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/linux/drm/xf86drm.c,v 1.35tsi Exp $ */
 
 #ifdef HAVE_XORG_CONFIG_H
 #include <xorg-config.h>
 #endif
 
-#include "xf86.h"
-#include "xf86_OSproc.h"
-#define _DRM_MALLOC xalloc
-#define _DRM_FREE   xfree
-
-#include "drm.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <ctype.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#define stat_t struct stat
-#include <sys/ioctl.h>
-#include <sys/mman.h>
-#include <sys/time.h>
-#include <stdarg.h>
+#ifdef XFree86Server
+# include "xf86.h"
+# include "xf86_OSproc.h"
+# include "drm.h"
+# include "xf86_ansic.h"
+# define _DRM_MALLOC xalloc
+# define _DRM_FREE   xfree
+# ifndef XFree86LOADER
+#  include <sys/mman.h>
+# endif
+#else
+# include <stdio.h>
+# include <stdlib.h>
+# include <unistd.h>
+# include <string.h>
+# include <ctype.h>
+# include <fcntl.h>
+# include <errno.h>
+# include <signal.h>
+# include <sys/types.h>
+# include <sys/stat.h>
+# define stat_t struct stat
+# include <sys/ioctl.h>
+# include <sys/mman.h>
+# include <sys/time.h>
+# include <stdarg.h>
+# ifdef DRM_USE_MALLOC
+#  define _DRM_MALLOC malloc
+#  define _DRM_FREE   free
+extern int xf86InstallSIGIOHandler(int fd, void (*f)(int, void *), void *);
+extern int xf86RemoveSIGIOHandler(int fd);
+# else
+#  include <X11/Xlibint.h>
+#  define _DRM_MALLOC Xmalloc
+#  define _DRM_FREE   Xfree
+# endif
+# include "drm.h"
+#endif
 
 /* No longer needed with CVS kernel modules on alpha 
 #if defined(__alpha__) && defined(__linux__)
@@ -72,7 +90,7 @@ extern unsigned long _bus_base(void);
 
 #include "xf86drm.h"
 
-#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#ifdef __FreeBSD__
 #define DRM_MAJOR 145
 #endif
 
@@ -115,9 +133,19 @@ drmMsg(const char *format, ...)
 {
     va_list	ap;
 
-    va_start(ap, format);
-    xf86VDrvMsgVerb(-1, X_NONE, DRM_MSG_VERBOSITY, format, ap);
-    va_end(ap);
+#ifndef XFree86Server
+    const char *env;
+    if ((env = getenv("LIBGL_DEBUG")) && strstr(env, "verbose"))
+#endif
+    {
+	va_start(ap, format);
+#ifdef XFree86Server
+	xf86VDrvMsgVerb(-1, X_NONE, DRM_MSG_VERBOSITY, format, ap);
+#else
+	vfprintf(stderr, format, ap);
+#endif
+	va_end(ap);
+    }
 }
 
 static void *drmHashTable = NULL; /* Context switch callbacks */
@@ -251,15 +279,19 @@ static int drmOpenDevice(long dev, int minor)
     int             fd;
     mode_t          devmode = DRM_DEV_MODE;
     int             isroot  = !geteuid();
+#if defined(XFree86Server)
     uid_t           user    = DRM_DEV_UID;
     gid_t           group   = DRM_DEV_GID;
+#endif
 
     sprintf(buf, DRM_DEV_NAME, DRM_DIR_NAME, minor);
     drmMsg("drmOpenDevice: node name is %s\n", buf);
 
+#if defined(XFree86Server)
     devmode  = xf86ConfigDRI.mode ? xf86ConfigDRI.mode : DRM_DEV_MODE;
     devmode &= ~(S_IXUSR|S_IXGRP|S_IXOTH);
     group = (xf86ConfigDRI.group >= 0) ? xf86ConfigDRI.group : DRM_DEV_GID;
+#endif
 
     if (stat(DRM_DIR_NAME, &st)) {
 	if (!isroot) return DRM_ERR_NOT_ROOT;
@@ -274,8 +306,10 @@ static int drmOpenDevice(long dev, int minor)
 	remove(buf);
 	mknod(buf, S_IFCHR | devmode, dev);
     }
+#if defined(XFree86Server)
     chown(buf, user, group);
     chmod(buf, devmode);
+#endif
 
     fd = open(buf, O_RDWR, 0);
     drmMsg("drmOpenDevice: open result is %d, (%s)\n",
@@ -289,8 +323,10 @@ static int drmOpenDevice(long dev, int minor)
 	if (!isroot) return DRM_ERR_NOT_ROOT;
 	remove(buf);
 	mknod(buf, S_IFCHR | devmode, dev);
+#if defined(XFree86Server)
 	chown(buf, user, group);
 	chmod(buf, devmode);
+#endif
     }
     fd = open(buf, O_RDWR, 0);
     drmMsg("drmOpenDevice: open result is %d, (%s)\n",
@@ -427,12 +463,16 @@ static int drmOpenByName(const char *name)
     char *        id;
     
     if (!drmAvailable()) {
+#if !defined(XFree86Server)
+	return -1;
+#else
         /* try to load the kernel module now */
         if (!xf86LoadKernelModule(name)) {
             ErrorF("[drm] failed to load kernel module \"%s\"\n",
 		   name);
             return -1;
         }
+#endif
     }
 
     /*
@@ -515,6 +555,7 @@ static int drmOpenByName(const char *name)
  */
 int drmOpen(const char *name, const char *busid)
 {
+#ifdef XFree86Server
     if (!drmAvailable() && name != NULL) {
 	/* try to load the kernel */
 	if (!xf86LoadKernelModule(name)) {
@@ -523,6 +564,7 @@ int drmOpen(const char *name, const char *busid)
 	    return -1;
 	}
     }
+#endif
 
     if (busid) {
 	int fd;
@@ -852,7 +894,7 @@ int drmAddMap(int fd,
     map.type    = type;
     map.flags   = flags;
     if (ioctl(fd, DRM_IOCTL_ADD_MAP, &map)) return -errno;
-    if (handle) *handle = (drm_handle_t)(unsigned long)map.handle;
+    if (handle) *handle = (drm_handle_t)map.handle;
     return 0;
 }
 
@@ -860,7 +902,7 @@ int drmRmMap(int fd, drm_handle_t handle)
 {
     drm_map_t map;
 
-    map.handle = (void *)(unsigned long)handle;
+    map.handle = (void *)handle;
 
     if(ioctl(fd, DRM_IOCTL_RM_MAP, &map)) return -errno;
     return 0;
@@ -1463,7 +1505,7 @@ int drmAgpEnable(int fd, unsigned long mode)
  * arguments in a drm_agp_buffer structure.
  */
 int drmAgpAlloc(int fd, unsigned long size, unsigned long type,
-		unsigned long *address, drm_handle_t *handle)
+		unsigned long *address, unsigned long *handle)
 {
     drm_agp_buffer_t b;
 
@@ -1490,7 +1532,7 @@ int drmAgpAlloc(int fd, unsigned long size, unsigned long type,
  * This function is a wrapper around the DRM_IOCTL_AGP_FREE ioctl, passing the
  * argument in a drm_agp_buffer structure.
  */
-int drmAgpFree(int fd, drm_handle_t handle)
+int drmAgpFree(int fd, unsigned long handle)
 {
     drm_agp_buffer_t b;
 
@@ -1514,7 +1556,7 @@ int drmAgpFree(int fd, drm_handle_t handle)
  * This function is a wrapper around the DRM_IOCTL_AGP_BIND ioctl, passing the
  * argument in a drm_agp_binding structure.
  */
-int drmAgpBind(int fd, drm_handle_t handle, unsigned long offset)
+int drmAgpBind(int fd, unsigned long handle, unsigned long offset)
 {
     drm_agp_binding_t b;
 
@@ -1537,7 +1579,7 @@ int drmAgpBind(int fd, drm_handle_t handle, unsigned long offset)
  * This function is a wrapper around the DRM_IOCTL_AGP_UNBIND ioctl, passing
  * the argument in a drm_agp_binding structure.
  */
-int drmAgpUnbind(int fd, drm_handle_t handle)
+int drmAgpUnbind(int fd, unsigned long handle)
 {
     drm_agp_binding_t b;
 
@@ -1727,7 +1769,7 @@ unsigned int drmAgpDeviceId(int fd)
     return i.id_device;
 }
 
-int drmScatterGatherAlloc(int fd, unsigned long size, drm_handle_t *handle)
+int drmScatterGatherAlloc(int fd, unsigned long size, unsigned long *handle)
 {
     drm_scatter_gather_t sg;
 
@@ -1739,7 +1781,7 @@ int drmScatterGatherAlloc(int fd, unsigned long size, drm_handle_t *handle)
     return 0;
 }
 
-int drmScatterGatherFree(int fd, drm_handle_t handle)
+int drmScatterGatherFree(int fd, unsigned long handle)
 {
     drm_scatter_gather_t sg;
 
@@ -1906,7 +1948,7 @@ int drmAddContextPrivateMapping(int fd, drm_context_t ctx_id, drm_handle_t handl
     drm_ctx_priv_map_t map;
 
     map.ctx_id = ctx_id;
-    map.handle = (void *)(unsigned long)handle;
+    map.handle = (void *)handle;
 
     if (ioctl(fd, DRM_IOCTL_SET_SAREA_CTX, &map)) return -errno;
     return 0;
@@ -1919,7 +1961,7 @@ int drmGetContextPrivateMapping(int fd, drm_context_t ctx_id, drm_handle_t * han
     map.ctx_id = ctx_id;
 
     if (ioctl(fd, DRM_IOCTL_GET_SAREA_CTX, &map)) return -errno;
-    if (handle) *handle = (drm_handle_t)(unsigned long)map.handle;
+    if (handle) *handle = (drm_handle_t)map.handle;
 
     return 0;
 }
@@ -1936,7 +1978,7 @@ int drmGetMap(int fd, int idx, drm_handle_t *offset, drmSize *size,
     *size   = map.size;
     *type   = map.type;
     *flags  = map.flags;
-    *handle = (drm_handle_t)(unsigned long)map.handle;
+    *handle = (unsigned long)map.handle;
     *mtrr   = map.mtrr;
     return 0;
 }
@@ -1959,7 +2001,7 @@ int drmGetClient(int fd, int idx, int *auth, int *pid, int *uid,
 int drmGetStats(int fd, drmStatsT *stats)
 {
     drm_stats_t s;
-    unsigned int i;
+    int         i;
 
     if (ioctl(fd, DRM_IOCTL_GET_STATS, &s)) return -errno;
 
@@ -2226,6 +2268,7 @@ int drmCommandWriteRead(int fd, unsigned long drmCommandIndex,
     return 0;
 }
 
+#if defined(XFree86Server) || defined(DRM_USE_MALLOC)
 static void drmSIGIOHandler(int interrupt, void *closure)
 {
     unsigned long key;
@@ -2289,3 +2332,4 @@ int drmRemoveSIGIOHandler(int fd)
 
     return xf86RemoveSIGIOHandler(fd);
 }
+#endif
