@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/xaa/xaaInit.c,v 1.34 2001/07/19 14:19:42 tsi Exp $ */
 
 #ifdef HAVE_XORG_CONFIG_H
 #include <xorg-config.h>
@@ -5,6 +6,7 @@
 
 #include "misc.h"
 #include "xf86.h"
+#include "xf86_ansic.h"
 #include "xf86_OSproc.h"
 
 #include <X11/X.h>
@@ -19,9 +21,6 @@
 #include "xaawrap.h"
 #include "xf86fbman.h"
 #include "servermd.h"
-#ifdef COMPOSITE
-#include "cw.h"
-#endif
 
 #define MAX_PREALLOC_MEM	65536	/* MUST be >= 1024 */
 
@@ -33,50 +32,57 @@ static void XAAGetImage(DrawablePtr pDrawable, int sx, int sy, int w, int h,
 			char *pdstLine);
 static void XAAGetSpans(DrawablePtr pDrawable, int wMax, DDXPointPtr ppt,
 			int *pwidth, int nspans, char *pdstStart);
-static PixmapPtr XAACreatePixmap(ScreenPtr pScreen, int w, int h, int depth,
-				 unsigned usage_hint);
+static PixmapPtr XAACreatePixmap(ScreenPtr pScreen, int w, int h, int depth);
 static Bool XAADestroyPixmap(PixmapPtr pPixmap);
+static void XAARestoreAreas (PixmapPtr pPixmap, RegionPtr prgnRestore, 
+			int xorg, int yorg, WindowPtr pWin);
+static void XAASaveAreas (PixmapPtr pPixmap, RegionPtr prgnSave, 
+			int xorg, int yorg, WindowPtr pWin);
 static Bool XAAEnterVT (int index, int flags);
 static void XAALeaveVT (int index, int flags);
 static int  XAASetDGAMode(int index, int num, DGADevicePtr devRet);
 static void XAAEnableDisableFBAccess (int index, Bool enable);
 static Bool XAAChangeWindowAttributes (WindowPtr pWin, unsigned long mask);
 
-static DevPrivateKeyRec XAAScreenKeyRec;
-#define XAAScreenKey (&XAAScreenKeyRec)
-static DevPrivateKeyRec XAAGCKeyRec;
-#define XAAGCKey (&XAAGCKeyRec)
-static DevPrivateKeyRec XAAPixmapKeyRec;
-#define XAAPixmapKey (&XAAPixmapKeyRec)
+/*
+ * XXX These three should be static, but that breaks ABI compat with XF4.4
+ * and Xorg 6.7.0 modules.  DO NOT use them in new code, you should never
+ * be setting them, and you've got Get functions below.
+ */
+int XAAScreenIndex = -1;
+int XAAGCIndex = -1;
+int XAAPixmapIndex = -1;
 
-DevPrivateKey XAAGetScreenKey(void) {
-    return XAAScreenKey;
+static unsigned long XAAGeneration = 0;
+
+int XAAGetScreenIndex(void) {
+    return XAAScreenIndex;
 }
 
-DevPrivateKey XAAGetGCKey(void) {
-    return XAAGCKey;
+int XAAGetGCIndex(void) {
+    return XAAGCIndex;
 }
 
-DevPrivateKey XAAGetPixmapKey(void) {
-    return XAAPixmapKey;
+int XAAGetPixmapIndex(void) {
+    return XAAPixmapIndex;
 }
 
 /* temp kludge */
 static Bool SwitchedOut = FALSE;
 
 XAAInfoRecPtr
-XAACreateInfoRec(void)
+XAACreateInfoRec()
 {
     XAAInfoRecPtr infoRec;
 
-    infoRec = calloc(1, sizeof(XAAInfoRec));
+    infoRec = xcalloc(1, sizeof(XAAInfoRec));
     if(infoRec)
 	infoRec->CachePixelGranularity = -1;
 
     return infoRec;
 }
 
-void
+void 
 XAADestroyInfoRec(XAAInfoRecPtr infoRec)
 {
     if(!infoRec) return;
@@ -84,39 +90,49 @@ XAADestroyInfoRec(XAAInfoRecPtr infoRec)
     if(infoRec->ClosePixmapCache)
 	(*infoRec->ClosePixmapCache)(infoRec->pScrn->pScreen);
    
-    free(infoRec->PreAllocMem);
+    if(infoRec->PreAllocMem)
+	xfree(infoRec->PreAllocMem);
 
-    free(infoRec->PixmapCachePrivate);
+    if(infoRec->PixmapCachePrivate)
+	xfree(infoRec->PixmapCachePrivate);
 
-    free(infoRec);
+    xfree(infoRec);
 }
 
 
-Bool
+Bool 
 XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
 {
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     XAAScreenPtr pScreenPriv;
     int i;
+#ifdef RENDER
     PictureScreenPtr    ps = GetPictureScreenIfSet(pScreen);
+#endif
 
     /* Return successfully if no acceleration wanted */
     if (!infoRec)
 	return TRUE;
     
-    if (!dixRegisterPrivateKey(&XAAGCKeyRec, PRIVATE_GC, sizeof(XAAGCRec)))
+    if (XAAGeneration != serverGeneration) {
+	if (	((XAAScreenIndex = AllocateScreenPrivateIndex()) < 0) ||
+		((XAAGCIndex = AllocateGCPrivateIndex()) < 0) ||
+		((XAAPixmapIndex = AllocatePixmapPrivateIndex()) < 0))
+		return FALSE;
+
+	XAAGeneration = serverGeneration;
+    }
+
+    if (!AllocateGCPrivate(pScreen, XAAGCIndex, sizeof(XAAGCRec)))
 	return FALSE;
 
-    if (!dixRegisterPrivateKey(&XAAPixmapKeyRec, PRIVATE_PIXMAP, sizeof(XAAPixmapRec)))
+    if (!AllocatePixmapPrivate(pScreen, XAAPixmapIndex, sizeof(XAAPixmapRec)))
 	return FALSE;
 
-    if (!dixRegisterPrivateKey(&XAAScreenKeyRec, PRIVATE_SCREEN, 0))
+    if (!(pScreenPriv = xalloc(sizeof(XAAScreenRec))))
 	return FALSE;
 
-    if (!(pScreenPriv = malloc(sizeof(XAAScreenRec))))
-	return FALSE;
-
-    dixSetPrivate(&pScreen->devPrivates, XAAScreenKey, pScreenPriv);
+    pScreen->devPrivates[XAAScreenIndex].ptr = (pointer)pScreenPriv;
 
     if(!xf86FBManagerRunning(pScreen))
 	infoRec->Flags &= ~(PIXMAP_CACHE | OFFSCREEN_PIXMAPS);
@@ -142,8 +158,16 @@ XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
 	infoRec->GetImage = XAAGetImage;
     if(!infoRec->GetSpans)
 	infoRec->GetSpans = XAAGetSpans;
+    if(!infoRec->PaintWindowBackground)
+	infoRec->PaintWindowBackground = XAAPaintWindow;
+    if(!infoRec->PaintWindowBorder)
+	infoRec->PaintWindowBorder = XAAPaintWindow;
     if(!infoRec->CopyWindow)
 	infoRec->CopyWindow = XAACopyWindow;
+    if(!infoRec->SaveAreas)
+	infoRec->SaveAreas = XAASaveAreas;
+    if(!infoRec->RestoreAreas)
+	infoRec->RestoreAreas = XAARestoreAreas;
 
     pScreenPriv->CreateGC = pScreen->CreateGC;
     pScreen->CreateGC = XAACreateGC;
@@ -153,12 +177,22 @@ XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
     pScreen->GetImage = infoRec->GetImage;
     pScreenPriv->GetSpans = pScreen->GetSpans;
     pScreen->GetSpans = infoRec->GetSpans;
+    pScreenPriv->PaintWindowBackground = pScreen->PaintWindowBackground;
+    pScreen->PaintWindowBackground = infoRec->PaintWindowBackground;
+    pScreenPriv->PaintWindowBorder = pScreen->PaintWindowBorder;
+    pScreen->PaintWindowBorder = infoRec->PaintWindowBorder;
     pScreenPriv->CopyWindow = pScreen->CopyWindow;
     pScreen->CopyWindow = infoRec->CopyWindow;
     pScreenPriv->CreatePixmap = pScreen->CreatePixmap;
     pScreen->CreatePixmap = XAACreatePixmap;
     pScreenPriv->DestroyPixmap = pScreen->DestroyPixmap;
     pScreen->DestroyPixmap = XAADestroyPixmap;
+    pScreenPriv->BackingStoreFuncs.RestoreAreas = 
+			pScreen->BackingStoreFuncs.RestoreAreas;
+    pScreen->BackingStoreFuncs.RestoreAreas = infoRec->RestoreAreas;
+    pScreenPriv->BackingStoreFuncs.SaveAreas = 
+			pScreen->BackingStoreFuncs.SaveAreas;
+    pScreen->BackingStoreFuncs.SaveAreas = infoRec->SaveAreas;
     pScreenPriv->ChangeWindowAttributes = pScreen->ChangeWindowAttributes;
     pScreen->ChangeWindowAttributes = XAAChangeWindowAttributes;
 
@@ -172,6 +206,7 @@ XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
     pScrn->EnableDisableFBAccess = XAAEnableDisableFBAccess;
 
     pScreenPriv->WindowExposures = pScreen->WindowExposures;
+#ifdef RENDER
     if (ps)
     {
 	pScreenPriv->Composite = ps->Composite;
@@ -179,10 +214,11 @@ XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
 	pScreenPriv->Glyphs = ps->Glyphs;
 	ps->Glyphs = XAAGlyphs;
     }
+#endif    
     if(pScrn->overlayFlags & OVERLAY_8_32_PLANAR)
         XAASetupOverlay8_32Planar(pScreen);
 
-    infoRec->PreAllocMem = malloc(MAX_PREALLOC_MEM);
+    infoRec->PreAllocMem = xalloc(MAX_PREALLOC_MEM);
     if(infoRec->PreAllocMem)
     	infoRec->PreAllocSize = MAX_PREALLOC_MEM;
 
@@ -193,14 +229,6 @@ XAAInit(ScreenPtr pScreen, XAAInfoRecPtr infoRec)
     if(infoRec->Flags & MICROSOFT_ZERO_LINE_BIAS)
 	miSetZeroLineBias(pScreen, OCTANT1 | OCTANT2 | OCTANT3 | OCTANT4);
 
-#ifdef COMPOSITE
-    /* Initialize the composite wrapper.  This needs to happen after the
-     * wrapping above (so it comes before us), but before all other extensions,
-     * so it doesn't confuse them. (particularly damage).
-     */
-    miInitializeCompositeWrapper(pScreen);
-#endif
-
     return TRUE;
 }
 
@@ -211,7 +239,7 @@ XAACloseScreen (int i, ScreenPtr pScreen)
 {
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     XAAScreenPtr pScreenPriv = 
-	(XAAScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XAAScreenKey);
+	(XAAScreenPtr) pScreen->devPrivates[XAAScreenIndex].ptr;
 
     pScrn->EnterVT = pScreenPriv->EnterVT; 
     pScrn->LeaveVT = pScreenPriv->LeaveVT; 
@@ -221,15 +249,21 @@ XAACloseScreen (int i, ScreenPtr pScreen)
     pScreen->CloseScreen = pScreenPriv->CloseScreen;
     pScreen->GetImage = pScreenPriv->GetImage;
     pScreen->GetSpans = pScreenPriv->GetSpans;
+    pScreen->PaintWindowBackground = pScreenPriv->PaintWindowBackground;
+    pScreen->PaintWindowBorder = pScreenPriv->PaintWindowBorder;
     pScreen->CopyWindow = pScreenPriv->CopyWindow;
     pScreen->WindowExposures = pScreenPriv->WindowExposures;
     pScreen->CreatePixmap = pScreenPriv->CreatePixmap;
     pScreen->DestroyPixmap = pScreenPriv->DestroyPixmap;
+    pScreen->BackingStoreFuncs.RestoreAreas = 
+			pScreenPriv->BackingStoreFuncs.RestoreAreas;
+    pScreen->BackingStoreFuncs.SaveAreas = 
+			pScreenPriv->BackingStoreFuncs.SaveAreas;
     pScreen->ChangeWindowAttributes = pScreenPriv->ChangeWindowAttributes;
 
     /* We leave it up to the client to free the XAAInfoRec */
 
-    free((pointer) pScreenPriv);
+    xfree ((pointer) pScreenPriv);
 
     return (*pScreen->CloseScreen) (i, pScreen);
 }
@@ -291,6 +325,137 @@ XAAGetSpans (
 }
 
 
+static void
+XAASaveAreas (
+    PixmapPtr pPixmap,
+    RegionPtr prgnSave,
+    int       xorg,
+    int       yorg,
+    WindowPtr pWin
+){
+    ScreenPtr pScreen = pPixmap->drawable.pScreen;
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    XAAInfoRecPtr infoRec = GET_XAAINFORECPTR_FROM_SCREEN(pScreen);
+
+    if(IS_OFFSCREEN_PIXMAP(pPixmap)) {
+	BoxPtr pbox = REGION_RECTS(prgnSave);
+	int nboxes = REGION_NUM_RECTS(prgnSave);
+
+	(*infoRec->SetupForScreenToScreenCopy)(pScrn, 1, 1, GXcopy, ~0, -1);
+	while(nboxes--) {
+	    (*infoRec->SubsequentScreenToScreenCopy)(pScrn, 
+		pbox->x1 + xorg, pbox->y1 + yorg, 
+		pPixmap->drawable.x + pbox->x1, 
+		pPixmap->drawable.y + pbox->y1,
+		pbox->x2 - pbox->x1, pbox->y2 - pbox->y1);
+	    pbox++;
+	}
+	SET_SYNC_FLAG(infoRec);
+	return;
+    }
+
+    if(xf86Screens[pScreen->myNum]->vtSema && infoRec->ReadPixmap &&
+	(pWin->drawable.bitsPerPixel == pPixmap->drawable.bitsPerPixel)) {
+	BoxPtr pbox = REGION_RECTS(prgnSave);
+	int nboxes = REGION_NUM_RECTS(prgnSave);
+	int Bpp =  pPixmap->drawable.bitsPerPixel >> 3;
+	unsigned char *dstp = (unsigned char*)pPixmap->devPrivate.ptr;
+
+	while(nboxes--) {
+	    (*infoRec->ReadPixmap)(infoRec->pScrn,
+		pbox->x1 + xorg, pbox->y1 + yorg, 
+		pbox->x2 - pbox->x1, pbox->y2 - pbox->y1, 
+		dstp + (pPixmap->devKind * pbox->y1) + (pbox->x1 * Bpp),
+		pPixmap->devKind,
+		pPixmap->drawable.bitsPerPixel, pPixmap->drawable.depth);
+	    pbox++;
+	}
+	return;
+    }
+
+    XAA_SCREEN_PROLOGUE (pScreen, BackingStoreFuncs.SaveAreas);
+    if(pScrn->vtSema) {
+	SYNC_CHECK(&pWin->drawable);
+    }
+    (*pScreen->BackingStoreFuncs.SaveAreas) (
+		pPixmap, prgnSave, xorg, yorg, pWin);
+
+    XAA_SCREEN_EPILOGUE (pScreen, BackingStoreFuncs.SaveAreas,
+			 XAASaveAreas);
+}
+
+static void
+XAARestoreAreas (    
+    PixmapPtr pPixmap,
+    RegionPtr prgnRestore,
+    int       xorg,
+    int       yorg,
+    WindowPtr pWin 
+){
+    ScreenPtr pScreen = pPixmap->drawable.pScreen;
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    XAAInfoRecPtr infoRec = GET_XAAINFORECPTR_FROM_SCREEN(pScreen);
+
+    if(IS_OFFSCREEN_PIXMAP(pPixmap)) {
+	BoxPtr pbox = REGION_RECTS(prgnRestore);
+	int nboxes = REGION_NUM_RECTS(prgnRestore);
+	int pm = ~0;
+
+	if((pScrn->overlayFlags & OVERLAY_8_32_PLANAR) && 
+					(pWin->drawable.depth == 24))
+	   pm = 0x00ffffff;
+
+	(*infoRec->SetupForScreenToScreenCopy)(pScrn, 1, 1, GXcopy, pm, -1);
+	while(nboxes--) {
+	    (*infoRec->SubsequentScreenToScreenCopy)(pScrn, 
+		pPixmap->drawable.x + pbox->x1 - xorg, 
+		pPixmap->drawable.y + pbox->y1 - yorg,
+		pbox->x1, pbox->y1, pbox->x2 - pbox->x1, pbox->y2 - pbox->y1);
+	    pbox++;
+	}
+	SET_SYNC_FLAG(infoRec);
+	return;
+    }
+
+    if(pScrn->vtSema && infoRec->WritePixmap &&
+	!(infoRec->WritePixmapFlags & NO_GXCOPY) &&
+	((pWin->drawable.bitsPerPixel == pPixmap->drawable.bitsPerPixel) ||
+		((pWin->drawable.bitsPerPixel == 24) &&  
+		 (pPixmap->drawable.bitsPerPixel == 32) &&
+		 (infoRec->WritePixmapFlags & CONVERT_32BPP_TO_24BPP)))) {
+	BoxPtr pbox = REGION_RECTS(prgnRestore);
+	int nboxes = REGION_NUM_RECTS(prgnRestore);
+	int Bpp =  pPixmap->drawable.bitsPerPixel >> 3;
+	unsigned char *srcp = (unsigned char*)pPixmap->devPrivate.ptr;
+	int pm = ~0;
+
+	if((pScrn->overlayFlags & OVERLAY_8_32_PLANAR) && 
+					(pWin->drawable.depth == 24))
+	   pm = 0x00ffffff;
+ 
+	while(nboxes--) {
+	    (*infoRec->WritePixmap)(pScrn, pbox->x1, pbox->y1, 
+		pbox->x2 - pbox->x1, pbox->y2 - pbox->y1, 
+		srcp + (pPixmap->devKind * (pbox->y1 - yorg)) + 
+				((pbox->x1 - xorg) * Bpp), 
+		pPixmap->devKind, GXcopy, pm, -1, 
+		pPixmap->drawable.bitsPerPixel, pPixmap->drawable.depth);
+	    pbox++;
+	}
+	return;
+    }
+
+    XAA_SCREEN_PROLOGUE (pScreen, BackingStoreFuncs.RestoreAreas);
+    if(pScrn->vtSema) {
+	SYNC_CHECK(&pWin->drawable);
+    }
+    (*pScreen->BackingStoreFuncs.RestoreAreas) (
+		pPixmap, prgnRestore, xorg, yorg, pWin);
+
+    XAA_SCREEN_EPILOGUE (pScreen, BackingStoreFuncs.RestoreAreas,
+				 XAARestoreAreas);
+}
+
 static int
 XAAPixmapBPP (ScreenPtr pScreen, int depth)
 {
@@ -299,8 +464,7 @@ XAAPixmapBPP (ScreenPtr pScreen, int depth)
     DestroyPixmapProcPtr    destroyPixmap;
     
     XAA_SCREEN_PROLOGUE (pScreen, CreatePixmap);
-    pPix = (*pScreen->CreatePixmap) (pScreen, 1, 1, depth,
-				     CREATE_PIXMAP_USAGE_SCRATCH);
+    pPix = (*pScreen->CreatePixmap) (pScreen, 1, 1, depth);
     XAA_SCREEN_EPILOGUE (pScreen, CreatePixmap, XAACreatePixmap);
     if (!pPix)
 	return 0;
@@ -331,23 +495,18 @@ XAAInitializeOffscreenDepths (ScreenPtr pScreen)
 }
 
 static PixmapPtr 
-XAACreatePixmap(ScreenPtr pScreen, int w, int h, int depth, unsigned usage_hint)
+XAACreatePixmap(ScreenPtr pScreen, int w, int h, int depth)
 {
     XAAInfoRecPtr infoRec = GET_XAAINFORECPTR_FROM_SCREEN(pScreen);
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     XAAPixmapPtr pPriv;
     PixmapPtr pPix = NULL;
     int size = w * h;
-
-    if (w > 32767 || h > 32767)
-	return NullPixmap;
     
     if (!infoRec->offscreenDepthsInitialized)
 	XAAInitializeOffscreenDepths (pScreen);
 
-    if(pScrn->vtSema &&
-	(usage_hint != CREATE_PIXMAP_USAGE_GLYPH_PICTURE) &&
-	(infoRec->offscreenDepths & (1 << (depth - 1))) &&
+    if(pScrn->vtSema && (infoRec->offscreenDepths & (1 << (depth - 1))) &&
 	(size >= MIN_OFFPIX_SIZE) && !SwitchedOut &&
 	(!infoRec->maxOffPixWidth || (w <= infoRec->maxOffPixWidth)) &&
 	(!infoRec->maxOffPixHeight || (h <= infoRec->maxOffPixHeight)) )
@@ -373,17 +532,17 @@ XAACreatePixmap(ScreenPtr pScreen, int w, int h, int depth, unsigned usage_hint)
 	    goto BAILOUT;
 	}
 
-        if(!(pLink = malloc(sizeof(PixmapLink)))) {
+        if(!(pLink = xalloc(sizeof(PixmapLink)))) {
             xf86FreeOffscreenArea(area);
 	    goto BAILOUT;
 	}
 
 	XAA_SCREEN_PROLOGUE (pScreen, CreatePixmap);
-	pPix = (*pScreen->CreatePixmap) (pScreen, 0, 0, depth, usage_hint);
+	pPix = (*pScreen->CreatePixmap) (pScreen, 0, 0, depth);
 	XAA_SCREEN_EPILOGUE (pScreen, CreatePixmap, XAACreatePixmap);
 
 	if (!pPix) {
-	    free(pLink);
+	    xfree (pLink);
             xf86FreeOffscreenArea(area);
 	    goto BAILOUT;
 	}
@@ -411,7 +570,7 @@ XAACreatePixmap(ScreenPtr pScreen, int w, int h, int depth, unsigned usage_hint)
     }
 BAILOUT:
     XAA_SCREEN_PROLOGUE (pScreen, CreatePixmap);
-    pPix = (*pScreen->CreatePixmap) (pScreen, w, h, depth, usage_hint);
+    pPix = (*pScreen->CreatePixmap) (pScreen, w, h, depth);
     XAA_SCREEN_EPILOGUE (pScreen, CreatePixmap, XAACreatePixmap);
 
     if(pPix) {
@@ -437,7 +596,7 @@ XAADestroyPixmap(PixmapPtr pPix)
     if(pPix->refcnt == 1) {
         if(pPriv->flags & OFFSCREEN) {
 	    if(pPriv->flags & DGA_PIXMAP)
-	        free(pPriv->offscreenArea);
+	        xfree(pPriv->offscreenArea);
             else {
 	        FBAreaPtr area = pPriv->offscreenArea;
 		PixmapLinkPtr pLink = infoRec->OffscreenPixmaps;
@@ -455,12 +614,12 @@ XAADestroyPixmap(PixmapPtr pPix)
 
 	        xf86FreeOffscreenArea(area);
 	        pPriv->offscreenArea = NULL;
-	        free(pLink);
+	        xfree(pLink);
 	    } 
         }
 
         if(pPriv->freeData) { /* pixmaps that were once in video ram */
-	    free(pPix->devPrivate.ptr);
+	    xfree(pPix->devPrivate.ptr);
 	    pPix->devPrivate.ptr = NULL;
 	}
     }
@@ -508,26 +667,19 @@ XAAChangeWindowAttributes (WindowPtr pWin, unsigned long mask)
 static Bool 
 XAAEnterVT(int index, int flags)
 {
-    ScrnInfoPtr pScrn = xf86Screens[index];
-    Bool ret;
     ScreenPtr pScreen = screenInfo.screens[index];
     XAAScreenPtr pScreenPriv = 
-	(XAAScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XAAScreenKey);
+	(XAAScreenPtr) pScreen->devPrivates[XAAScreenIndex].ptr;
 
-    pScrn->EnterVT = pScreenPriv->EnterVT;
-    ret = ((*pScreenPriv->EnterVT)(index, flags));
-    pScreenPriv->EnterVT = pScrn->EnterVT;
-    pScrn->EnterVT = XAAEnterVT;
-    return ret;
+    return((*pScreenPriv->EnterVT)(index, flags));
 }
 
 static void 
 XAALeaveVT(int index, int flags)
 {
-    ScrnInfoPtr pScrn = xf86Screens[index];
     ScreenPtr pScreen = screenInfo.screens[index];
     XAAScreenPtr pScreenPriv = 
-	(XAAScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XAAScreenKey);
+	(XAAScreenPtr) pScreen->devPrivates[XAAScreenIndex].ptr;
     XAAInfoRecPtr infoRec = pScreenPriv->AccelInfoRec;
 
     if(infoRec->NeedToSync) {
@@ -535,10 +687,7 @@ XAALeaveVT(int index, int flags)
         infoRec->NeedToSync = FALSE;
     }
 
-    pScrn->LeaveVT = pScreenPriv->LeaveVT;
     (*pScreenPriv->LeaveVT)(index, flags);
-    pScreenPriv->LeaveVT = pScrn->LeaveVT;
-    pScrn->LeaveVT = XAALeaveVT;
 }
 
 typedef struct {
@@ -553,7 +702,7 @@ XAASetDGAMode(int index, int num, DGADevicePtr devRet)
     ScreenPtr pScreen = screenInfo.screens[index];
     XAAInfoRecPtr infoRec = GET_XAAINFORECPTR_FROM_SCREEN(pScreen);
     XAAScreenPtr pScreenPriv = 
-	(XAAScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XAAScreenKey);
+	(XAAScreenPtr) pScreen->devPrivates[XAAScreenIndex].ptr;
     int ret;
 
     if (!num && infoRec->dgaSaves) { /* restore old pixmap cache state */
@@ -562,7 +711,7 @@ XAASetDGAMode(int index, int num, DGADevicePtr devRet)
 	infoRec->UsingPixmapCache = state->UsingPixmapCache;	
 	infoRec->CanDoColor8x8 = state->CanDoColor8x8;	
 	infoRec->CanDoMono8x8 = state->CanDoMono8x8;
-	free(infoRec->dgaSaves);
+	xfree(infoRec->dgaSaves);
 	infoRec->dgaSaves = NULL;
     }
 
@@ -573,7 +722,7 @@ XAASetDGAMode(int index, int num, DGADevicePtr devRet)
 	XAAPixmapPtr pixPriv = XAA_GET_PIXMAP_PRIVATE(devRet->pPix);
 	FBAreaPtr area;
 
-	if((area = malloc(sizeof(FBArea)))) {
+	if((area = xalloc(sizeof(FBArea)))) {
 	    area->pScreen = pScreen;
 	    area->box.x1 = 0;
 	    area->box.x2 = 0;
@@ -588,7 +737,7 @@ XAASetDGAMode(int index, int num, DGADevicePtr devRet)
 	    pixPriv->offscreenArea = area;
 
 	    if(!infoRec->dgaSaves) { /* save pixmap cache state */
-		SavedCacheStatePtr state = malloc(sizeof(SavedCacheState));
+		SavedCacheStatePtr state = xalloc(sizeof(SavedCacheState));
 	
 		state->UsingPixmapCache = infoRec->UsingPixmapCache;	
 		state->CanDoColor8x8 = infoRec->CanDoColor8x8;	
@@ -615,7 +764,7 @@ XAAEnableDisableFBAccess (int index, Bool enable)
     ScreenPtr pScreen = screenInfo.screens[index];
     XAAInfoRecPtr infoRec = GET_XAAINFORECPTR_FROM_SCREEN(pScreen);
     XAAScreenPtr pScreenPriv = 
-	(XAAScreenPtr)dixLookupPrivate(&pScreen->devPrivates, XAAScreenKey);
+	(XAAScreenPtr) pScreen->devPrivates[XAAScreenIndex].ptr;
 
     if(!enable) {
 	if((infoRec->Flags & OFFSCREEN_PIXMAPS) && (infoRec->OffscreenPixmaps))
