@@ -1,3 +1,4 @@
+/* $Xorg: miline.h,v 1.4 2001/02/09 02:05:21 xorgcvs Exp $ */
 
 /*
 
@@ -24,11 +25,15 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
+/* $XFree86: xc/programs/Xserver/mi/miline.h,v 1.6 2001/08/06 20:51:19 dawes Exp $ */
+
+#ifdef HAVE_DIX_CONFIG_H
+#include <dix-config.h>
+#endif
 
 #ifndef MILINE_H
 
 #include "screenint.h"
-#include "privates.h"
 
 /*
  * Public definitions used for configuring basic pixelization aspects
@@ -77,7 +82,7 @@ in this Software without prior written authorization from The Open Group.
  * Jack E. Bresenham, IEEE CG&A, May 1987.
  */
 
-extern _X_EXPORT void miSetZeroLineBias(
+extern void miSetZeroLineBias(
     ScreenPtr /* pScreen */,
     unsigned int /* bias */
 );
@@ -110,7 +115,7 @@ extern _X_EXPORT void miSetZeroLineBias(
 }
   
 #define SWAPINT(i, j) \
-{  int _t = i;  i = j;  j = _t; }
+{  register int _t = i;  i = j;  j = _t; }
 
 #define SWAPPT(i, j) \
 {  DDXPointRec _t; _t = i;  i = j; j = _t; }
@@ -120,8 +125,9 @@ extern _X_EXPORT void miSetZeroLineBias(
         t = y1;  y1 = y2;  y2 = t;\
 }
 
-#define miGetZeroLineBias(_pScreen) ((unsigned long) (unsigned long*)\
-    dixLookupPrivate(&(_pScreen)->devPrivates, miZeroLineScreenKey))
+#define miGetZeroLineBias(_pScreen) \
+    ((miZeroLineScreenIndex < 0) ? \
+     		0 : ((_pScreen)->devPrivates[miZeroLineScreenIndex].uval))
 
 #define CalcLineDeltas(_x1,_y1,_x2,_y2,_adx,_ady,_sx,_sy,_SX,_SY,_octant) \
     (_octant) = 0;				\
@@ -148,9 +154,9 @@ extern _X_EXPORT void miSetZeroLineBias(
 #define IsXDecreasingOctant(_octant)	((_octant) & XDECREASING)
 #define IsYDecreasingOctant(_octant)	((_octant) & YDECREASING)
 
-extern _X_EXPORT DevPrivateKey miZeroLineScreenKey;
+extern int miZeroLineScreenIndex;
 
-extern _X_EXPORT int miZeroClipLine(
+extern int miZeroClipLine(
     int /*xmin*/,
     int /*ymin*/,
     int /*xmax*/,
